/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * HomologyContainerUnitTest.cpp
 *
 *  Created on: Sep 22, 2012
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

using namespace std;

#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>

// include headers that implement a archive in simple text format
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>

#include <boost/assign.hpp>

#include "Fragmentation/Homology/HomologyContainer.hpp"
#include "Fragmentation/Graph.hpp"

#include "HomologyContainerUnitTest.hpp"

#include <sstream>

using namespace boost::assign;

#ifdef HAVE_TESTRUNNER
#include "UnitTestMain.hpp"
#endif /*HAVE_TESTRUNNER*/

/********************************************** Test classes **************************************/

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( HomologyContainerTest );


void HomologyContainerTest::setUp()
{
  // add nodes
  nodes +=
      std::make_pair(FragmentNode(1,1),1),
      std::make_pair(FragmentNode(1,4),1),
      std::make_pair(FragmentNode(2,2),1),
      std::make_pair(FragmentNode(2,4),1);
  othernodes +=
      std::make_pair(FragmentNode(1,1),1),
      std::make_pair(FragmentNode(1,4),1);

  // add edges
  edges +=
      std::make_pair(FragmentEdge(1,1),1),
      std::make_pair(FragmentEdge(1,4),1),
      std::make_pair(FragmentEdge(2,2),1),
      std::make_pair(FragmentEdge(2,4),1);
  otheredges +=
      std::make_pair(FragmentEdge(1,4),1),
      std::make_pair(FragmentEdge(2,2),1);

  // construct graphs
  HomologyGraph graph(nodes, edges);
  HomologyGraph othergraph(othernodes, otheredges);

  // place in container
  Fragment::position_t pos(3, 0.);
  Fragment::positions_t positions(1, pos);
  Fragment::charges_t charges(1,1.);
  Fragment dummy(positions, charges);
  charges[0] = 6.;
  positions[0][0] = 1.;
  Fragment dummy1(positions, charges);
  positions[0][0] = 2.;
  Fragment dummy2(positions, charges);
  HomologyContainer::value_t value1;
  value1.fragment = dummy1;
  value1.energy = 1.;
  HomologyContainer::value_t value2;
  value2.fragment = dummy2;
  value2.energy = 1.5;
  HomologyContainer::value_t value3;
  value3.fragment = dummy;
  value3.energy = 2.;
  container +=
      std::make_pair( graph, value1 ),
      std::make_pair( graph, value2 ),
      std::make_pair( othergraph, value3 );
  // create HomologyContainer
  Keys = new HomologyContainer(container);
}


void HomologyContainerTest::tearDown()
{
  delete Keys;
}

/** UnitTest for whether homologies are correctly recognized
 */
void HomologyContainerTest::InsertionTest()
{
  // construct graphs
  HomologyGraph graph(nodes, edges);

  HomologyContainer::container_t newcontainer;
  Fragment::position_t pos(3, 0.);
  Fragment::positions_t positions(1, pos);
  Fragment::charges_t charges(1,1.);
  Fragment dummy(positions, charges);
  HomologyContainer::value_t value;
  value.fragment = dummy;
  value.energy = 1.;
  newcontainer +=
      std::make_pair( graph, value );

  Keys->insert(newcontainer);

  CPPUNIT_ASSERT_EQUAL( (size_t)4, Keys->container.size() );
}

/** UnitTest for operator==() works correctly.
 */
void HomologyContainerTest::EqualityTest()
{
  // compare same container
  CPPUNIT_ASSERT( *Keys == *Keys );

  // construct other container
  HomologyGraph graph(nodes, edges);
  HomologyContainer::container_t newcontainer;
  Fragment::position_t pos(3, 0.);
  Fragment::positions_t positions(1, pos);
  Fragment::charges_t charges(1,1.);
  Fragment dummy(positions, charges);
  HomologyContainer::value_t value;
  value.fragment = dummy;
  value.energy = 1.;
  newcontainer +=
      std::make_pair( graph, value );

  HomologyContainer other(newcontainer);

  CPPUNIT_ASSERT( *Keys != other );
}

/** UnitTest for serialization
 */
void HomologyContainerTest::serializeTest()
{
  // serialize
  std::stringstream outputstream;
  boost::archive::text_oarchive oa(outputstream);
  oa << Keys;

  // deserialize
  HomologyContainer *samekeys = NULL;
  std::stringstream returnstream(outputstream.str());
  boost::archive::text_iarchive ia(returnstream);
  ia >> samekeys;

  CPPUNIT_ASSERT( samekeys != NULL );
  //std::cout << *Keys << std::endl;
  CPPUNIT_ASSERT_EQUAL( *Keys, *samekeys );

  delete samekeys;
}
